#!/usr/bin/perl -w
# -T can only be given on the command line (taint checking)
use strict; # homework by Eric Auer...
# push(@INC,"."); # add local path

# Indexing over a set of HTML files
my $THERE = ".";  # where to start in directory tree...
my $logfile = "/dev/null";
my $DOWNCASE = 1; # set to one to force the index to lower case
my $MORPHO = 1;   # *** week6: set to 1 for simple morphology...
my $GZIP = 1;     # *** week6: set to 1 to gzip the index if
                  # *** gzip is installed (compresses well: lots
                  # *** of filenames repeated, lots of numbers)

require "index-browse.pl";
require "index-demarkup2.pl";
require "index-deflex.pl";   # *** for week 6, simple morphology

my %hsh;     # the big hash of hashes for the index

if ($THERE eq ".") {
  (chomp($THERE = `pwd`)) || ($THERE = "./");
  print STDERR "Search begins in $THERE\n";
  # I really miss a pwd function in Perl...
}

my @htmllist = split("\011",&collecthtmlfiles($THERE));

  # *** week 6: we now use tab as file name separator,
  # *** because (Win)DOS uses the : for drivespecs. sigh.

foreach my $onefile (@htmllist) {

  my $content = &onthtml($onefile);

  $content =~ s/[^\w\007]/ /gs;   # simplify "other" but "\007"
                               # non-word -> " "
  $content =~ s/[\s\n][\s\n]+/ /gs;   # compress spaces and \n

  my $pos = 0;

  foreach my $oneword (split(" ",$content)) {
    if ($oneword =~ /^_*$/) { next; } # skip "nowords"
    $pos++; # we store the word numbers where the word occurs
    if ($DOWNCASE == 1) {
      $oneword = lc($oneword);     # downcase if requested...
    }

    if ($MORPHO == 1) {
      $oneword = &deflex($oneword); # *** week6: morphology
    }

    if (defined $hsh{$oneword}->{$onefile}) {
      # add one more reference to that word in that file
      $hsh{$oneword}->{$onefile} = 
        $hsh{$oneword}->{$onefile} . "," . $pos;
    } else {
      # create a new sub-hash for that word in that file
      $hsh{$oneword}->{$onefile} = $pos;
    }
  }
  $hsh{"wordcount\007"}->{$onefile} = $pos;
    # store this for relevance ranking
}

open(LOG,">$logfile") || die "cannot write log file $logfile\n";

if ($GZIP == 1) {
  unless (open(OFILE,"|gzip -c >index.txt.gz")) {
    print STDERR "This system has no gzip or there was an write"
      .   "\nerror with index.txt.gz, reverting to plain\n";
    open(OFILE,">index.txt") || die "cannot write index.txt\n";
  }
} else {
  open(OFILE,">index.txt") || die "cannot write index.txt\n";
}

print LOG "Files indexed: " . scalar @htmllist . "\n"; # *** week 6:
print OFILE "" . scalar @htmllist . "\n"; # *** week 6:
 # *** write filecount to "header" of file

foreach my $oneword (sort keys %hsh) {
  printf LOG "%20s: ", $oneword;
  print OFILE "$oneword";
  foreach my $item (sort keys %{$hsh{$oneword}}) { # confusing,
                                                   # but works!
    print LOG "<\"$item\"," . $hsh{$oneword}->{$item} . "> ";
    print OFILE ";$item," . $hsh{$oneword}->{$item};
  }
  print LOG "\n";
  print OFILE "\n";
}

close(OFILE); # this will also close the piped progs like gzip
close(LOG);
