#!/usr/bin/perl -w
# -T can only be given on the command line (taint checking)
use strict; # homework by Eric Auer: a complex index query
            # tool with various gadgets (week 6 version 2/2002)

# push(@INC,"."); # add local path

my $USEIDF = 1;      # *** week 6: set to multiply weight by IDF
                     # *** -> log(num_of_docs/num_hit_docs)
my $USEMARKUP = 1;   # *** week 6: set to multiply weight by
                     # *** a measure of font size etc.
my $MORPHO = 1;      # *** week 6: set to 1 to enable morphology
my $SHOWHTML = 1;    # *** week 6: gadget: HTML + context output
                     # *** showcontext(file,words,hashref),
                     # *** showtitle(file)
my $BROWSER = "w3m"; # program used to open the html output

my $TESTQ = 0; # set for debugging output: 0 none, 1 set size
               # and relevant notifications,
               # 2 for set keys, 3 for keys and values
               # 2 also shows markedness and idf calculations

# this is what we should implement as the final task of week4
# (not counting the optional tasks): queries with +word and
# -word and word, weighting and multiple words.

require "query-read.pl"; # read the index into a %hash
# usage: %{$hsh{$word}} has elements $hsh{$word}->{$file}
# which are position strings like "2,5,11". We also have special
# words: "title:word" and "wordcount:"

require "query-sets.pl"; # setAND setOR setSUB setCOPY setWEIGH
# all operate on hashrefs, all with 2 args modify the first
# hash in place as well, set ops are done on keys, values
# are taken as numeric weights and modified as well

require "index-deflex.pl"; # *** week 6: simple morphology
require "query-show.pl";   # *** week 6: optional gadget:
                           # *** show context

my $oneword;
my %results;
my %stopwords = ("and",1,"und",1,"en",1);
my $adjust; # do not forget to set this before using findterm!


my %hsh = %{&reftoindex()};


sub setSHOW {
  if ($TESTQ < 1) { return 0; }
  if ($TESTQ < 2) {
    print scalar keys %{$_[0]}; # keys in scalar context give
    print "\n"; # size...
  } elsif ($TESTQ < 3) {
    print "(";
    print join(" ",(keys %{$_[0]}));
    print ")\n";
  } elsif ($TESTQ < 4) {
    my %theset = %{$_[0]};
    foreach my $thekey (keys %theset) { # no "keys" or "values"
                                        # -> k,v,k,v,...
      printf "%s ( %3.5f ) ", $thekey, $theset{$thekey};
    }
    print "\n";
  } else { print "setSHOW what???\n"; }
  return 1;
}


sub findterm { # returns hashref: keys are files, values weights
  my $word = $_[0];
  my %results = ();

  if (defined $stopwords{$word}) {
    print "You may not search for <$word>,"
     .    " that could lead to a huge list of hits\n";
    return \%results; # empty list
  }

  if ($MORPHO == 1) {
    my $oldword = $word;
    $word = &deflex($word);
    unless ($TESTQ < 1) {
      print "morphology: <$oldword> to <$word>\n"
        if ($oldword ne $word);
    }
  } # *** week 6

  if (defined $hsh{$word}) {
    unless ($TESTQ < 1) { print "$word was found\n"; };

    foreach my $onefile (keys %{$hsh{$word}}) {
      my @posn = split(/,/,$hsh{$word}->{$onefile});
      # print "In file $onefile, " . ($#posn+1) . " of "
      #  . $hsh{"wordcount:"}->{$onefile} . " times\n";
      $results{$onefile} = log(
        $adjust *
        ($#posn + 1) / $hsh{"wordcount:"}->{$onefile});

      # *** week 6 stuff follows
      if ($USEIDF) {    # multiply by log(docs/docs-w-that-term)
        $results{$onefile} *= (0.01 + log(&filesinindex /
        (scalar keys %{$hsh{$word}}) ));
        print "IDF for $word is: " . (0.01 + log(&filesinindex /
          (scalar keys %{$hsh{$word}}) )) . "\n"
          unless ($TESTQ < 2);
        # even a term found in all files has an IDF != 0
        # because we add 0.01 of bias
      }

      # *** week 6 stuff follows
      if ($USEMARKUP) { # multiply by prominence of term
        $results{$onefile} *= &markedness($word,$onefile);
      }

      # store result along with relevance (log relative
      # frequency)
    }
  } else {
    if ($word) {
      print "$word was not found\n";
    }
  }

  &setSHOW(\%results);
  return \%results;
}


sub markedness {
  my ($w,$f) = @_; # copy args: word, file
  my $factor = 1;
  my %marks = ("h1",2, "h2",1.5, "h3",1.25,
               "em",1.25, "i",1.25, "b", 1.25,
               "strong",1.25, "title",3);
     # the weight factors per markup type
  foreach my $how (keys %marks) { # try to find all variants
    if (defined $hsh{"${how}:$w"}->{$f}) {
      $factor *= $marks{$how};
      print "<$w> highlighted by $how: factor now $factor\n"
        unless ($TESTQ < 2);
    }
  }
  return $factor;
}


# --------------------------------------------------------------


$adjust = 0;
foreach $oneword (keys %{$hsh{"wordcount:"}}) {
  if (log($hsh{"wordcount:"}->{$oneword}) >
      $adjust) {
    $adjust = $hsh{"wordcount:"}->{$oneword};
  }
}
print "The longest file contains $adjust words\n";
$adjust *= exp(1.0);
# so log($adjust * freq / filesize) will be 1..something or so.


my $words;
do {
  %results = ();              # 0 pairs (empty list)
  print "You can use +word and -word to force/disallow terms\n";
  print "Search for which terms (space separated) ?\n";
  $words = <STDIN>;
  chomp $words;
  $words = lc($words);    # only lowercase terms
  my @andterms = ();
  my  @orterms = ();
  my @notterms = ();
  my $numand = 0;    # *** week 6 tuning

  foreach my $oneword (split(/ /,$words)) {
    if ($oneword =~ /^[+]/) {
      unshift(@andterms,substr($oneword,1));
      $numand++;
      # *** was wrong (fixed for week 6):
      # *** unshift(@orterms,substr($oneword,1));
    } elsif  ($oneword =~ /^[-]/) {
      unshift(@notterms,substr($oneword,1));
    } else {
      unshift(@orterms,$oneword);
    }
  }

  print "Checking " . (scalar @orterms) . " optional, "
    .   $numand . " required and "
    .   (scalar @notterms) . " blocked terms\n"
    unless ($TESTQ < 1);

  # *** fixed in week 6: AND terms go before OR terms!
  foreach my $oneword (@andterms) { # collect hits with +terms
    unless ($TESTQ < 1) {
      print "Adding files with <$oneword>\n";
    }
    # my %moreres = %{&findterm($oneword)}; # always search
    %results = %{&setOR(\%results,&findterm($oneword))};
    &setSHOW(\%results);
  }

  if ($numand > 1) {
    foreach my $oneword (@andterms) { # enforce ALL +words
      unless ($TESTQ < 1) {
        print "Require files to have <$oneword>\n";
      }
      %results = %{&setAND(\%results,&findterm($oneword))};
      &setSHOW(\%results);
    }
  }

  foreach my $oneword (@orterms) { # add words 
                                    # if ok with +words
    unless ($TESTQ < 1) {
      print "Require files to have <$oneword>\n";
    }
    if ($numand == 0) {
      %results = %{&setOR(\%results,&findterm($oneword))};
      # *** week 6: words can only add new files if we have
      # *** +words to care for!
    } else {
      %results = %{&setWEIGH(\%results,&findterm($oneword))};
      # *** does neither add nor remove files, but changes
      # *** the weighting (week 6)
    }
    &setSHOW(\%results);
  }

  foreach my $oneword (@notterms) { # enforce -words
    unless ($TESTQ < 1) {
      print "Exclude files with <$oneword>\n";
    }
    %results = %{&setSUB(\%results,&findterm($oneword))};
    &setSHOW(\%results);
  }

  if ($words) {
    print "Results by relevance:\n";
    foreach my $res (sort {$results{$b} <=> $results{$a}}
                     (keys(%results)) ) {
      printf "%-20s (%3.5f)\n", $res, $results{$res};
    }
    print scalar keys %results;
    print " file(s) matched the query\n\n";

    if (($SHOWHTML) && (scalar keys %results)) {
       # *** week 6: generate HTML presentation
       # *** skipped if no results were found
       open(HFILE,">engine-tmp.html")
         || die "cannot write HTMLized results\n";
       print HFILE "<html>\n<head>\n<title>Text Retrieval "
         . "2002 UU.NL Eric Auer: Results for $words\n"
         . "</title></head><body bgcolor=white text=black>\n"
         . "Search results for $words:<hr>\n"
         . "<ol>\n\n";

       foreach my $res (sort {$results{$b} <=> $results{$a}}
                        (keys(%results)) ) {
         # printf "%-20s (%3.5f)\n", $res, $results{$res};
         printf HFILE "<li> <b>(%3.5f", $results{$res};
         print HFILE ")</b>\n"
           .  "<a href=\"$res\">"
           .  &showtitle($res) . "</a><br>\n"
           .  "<small>($res)</small><br><br>\n"
           .  &showcontext($res,$words,&reftoindex)
           .  "\n<hr></li>\n\n";
       }

       print HFILE "\n\n</ol>\n<hr>\n\n" . scalar keys %results;
       print HFILE " file(s) matched the query\n"
         .  "</body>\n</html>\n";
       close(HFILE);
       system($BROWSER, "engine-tmp.html");
    }

  }

# print "Results for query <$words> completed.\n";
} while ($words);

